package com.mediterranium.locator;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import utils.Constants;

import com.mediterranium.planificacion.IPlanificacionFacade;
import com.mediterranium.planificacion.PlanificacionFacadeBean;
import com.mediterranium.user.IUserFacade;
import com.mediterranium.user.UserFacadeBean;
import com.metditerranium.receptes.IReceptesFacade;
import com.metditerranium.receptes.ReceptesFacadeBean;

public class EJBLocator {

	private static final String PKG_INTERFACES = "org.jboss.ejb.client.naming";
	private static Context initialContext;
	
	
	public static EJBLocator locatorUser;
	public static EJBLocator locatorReceptes;
	public static IUserFacade userFacade;
	public static IReceptesFacade receptesFacade;
	public static IPlanificacionFacade planificacionFacade;
	
	public EJBLocator(String ejb)throws NamingException{
		//inicializamos contexto 1 vez
		if (initialContext == null) {
            Properties properties = new Properties();
            properties.put(Context.URL_PKG_PREFIXES, PKG_INTERFACES);
 
            initialContext = new InitialContext(properties);
		}
		
		//si se busca la fachada de user
		if(ejb.equals("userFacade")){
			String lookupName = getLookupName(UserFacadeBean.class.getSimpleName(),
												IUserFacade.class.getName());
			
			userFacade = (IUserFacade) initialContext.lookup(lookupName);
		}
		if(ejb.equals(Constants.FACADE_RECETAS)){
			String lookupName = getLookupName(ReceptesFacadeBean.class.getSimpleName(),
												IReceptesFacade.class.getName());
			
			receptesFacade = (IReceptesFacade) initialContext.lookup(lookupName);
		}
		if(ejb.equals(Constants.FACADE_PLANIFICACION)){
			String lookupName = getLookupName(PlanificacionFacadeBean.class.getSimpleName(),
												IPlanificacionFacade.class.getName());
			
			planificacionFacade = (IPlanificacionFacade) initialContext.lookup(lookupName);
		}
	}
	
	public static EJBLocator getInstanceUser(String ejb) throws NamingException{
		if(userFacade==null){
			synchronized (EJBLocator.class) {
				if(userFacade==null)
					locatorUser = new EJBLocator(ejb);
				
			}
		} 
		return locatorUser;
	}
	
	public static EJBLocator getInstanceReceptes(String ejb) throws NamingException{
		if(receptesFacade==null){
			synchronized (EJBLocator.class) {
				if(receptesFacade==null)
					locatorReceptes = new EJBLocator(ejb);
				
			}
		} 
		return locatorReceptes;
	}
	
	public static EJBLocator getInstancePlanificacion(String ejb) throws NamingException{
		if(planificacionFacade==null){
			synchronized (EJBLocator.class) {
				if(planificacionFacade==null)
					locatorReceptes = new EJBLocator(ejb);
				
			}
		} 
		return locatorReceptes;
	}
	
	private static String getLookupName(String implClassName,String fullyQualifiedInterface) {
        /*The app name is the EAR name of the deployed EJB without .ear
        suffix. Since we haven't deployed the application as a .ear, the app
        name for us will be an empty string */
        String appName = "ear-1.0-SNAPSHOT";
 
        /* The module name is the JAR name of the deployed EJB without the
        .jar suffix.*/
        String moduleName = "business-logic-1.0-SNAPSHOT";//"";
 
        /* AS7 allows each deployment to have an (optional) distinct name.
        This can be an empty string if distinct name is not specified.*/
        String distinctName = "";
 
        // The EJB bean implementation class name
        String beanName = implClassName;
 
        // Fully qualified remote interface name
        final String interfaceName = fullyQualifiedInterface;
 
        // Create a look up string name
        String name = "ejb:" + appName + "/" + moduleName + "/" +
                distinctName    + "/" + beanName + "!" + interfaceName;
        return name;
    }
	
	public static IUserFacade getUserFacade() {
		return userFacade;
	}
	
	public static IReceptesFacade getReceptesFacade() {
		return receptesFacade;
	}
	
	public static IPlanificacionFacade getPlanificacionFacade() {
		return planificacionFacade;
	}
	
}
